package cn.com.qmth.examcloud.core.oe.student.service;


import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.core.oe.student.bean.*;
import cn.com.qmth.examcloud.support.enums.HandInExamType;

/**
 * @author chenken
 * @date 2018年8月13日 下午2:09:38
 * @company QMTH
 * @description 在线考试控制服务接口
 */
public interface ExamControlService {

    /**
     * 开始考试
     */
    StartExamInfo startExam(Long examStudentId, Long userId, String ip, Boolean allowOnline);

    /**
     * 开始考试（新）
     * 注：需要加、解密
     */
    String startExamWithCrypto(String encryptParams, User user, String requestIP, String timestampStr);

    /**
     * 开始答题
     */
    StartAnswerInfo startAnswer(Long examRecordDataId, Long userId);

    /**
     * 手动交卷
     */
    void manualEndExam(Long studentId, String ip, Boolean force);

    /**
     * 手动交卷（新）
     * 注：需要加、解密
     */
    void manualEndExamWithCrypto(String encryptParams, User user, String requestIP, String timestampStr);

    /**
     * 交卷
     *
     * @param examRecordDataId 考试记录id
     * @param handInExamType   交卷类型
     * @param ip               请求ip，可以为空
     */
    void handInExam(Long examRecordDataId, HandInExamType handInExamType, String ip, Boolean force);

    /**
     * 断点续考:检查正在进行中的考试
     *
     * @param studentId
     */
    CheckExamInProgressInfo checkExamInProgress(Long studentId, String ip);

    ExamProcessResultInfo checkExamInProgress2(Long studentId, String ip);

    /**
     * 考试心跳
     *
     * @param
     */
    long examHeartbeat(User user, String ip);

    void setAndSaveActiveTime(Long examRecordDataId, String ip);

    /**
     * 获取考试结束后的相关信息
     *
     * @param examRecordDataId
     * @return
     */
    EndExamInfo getEndExamInfo(Long examRecordDataId);

    CheckQrCodeInfo checkQrCode(String qrCode);

    /**
     * 发送消息到websocket
     *
     * @param examRecordDataId 考试记录id
     * @param order            题序号
     * @param fileUrl          文件路径
     * @param transferFileType 传输文件类型
     * @throws Exception
     */
    void sendFileAnswerToWebSocket(Long examRecordDataId, Integer order,
                                   String fileUrl, String transferFileType, Long userId, Long rootOrgId) throws Exception;

    /**
     * 通过websocket发送二维码扫描信息
     *
     * @param examRecordDataId
     * @param clientId
     * @throws Exception
     */
    void sendScanQrCodeToWebSocket(String clientId, Long examRecordDataId, Integer order,
                                   Long userId, Long rootOrgId) throws Exception;

    UpyunSignatureInfo getUpyunSignature(GetYunSignatureReq req);

    /**
     * 获取二维码
     *
     * @param req
     * @param user
     * @return
     */
    String getQrCode(GetQrCodeReq req, User user);

    /**
     * 记录切换屏幕次数
     *
     * @param examRecordDataId
     */
    SwitchScreenCountInfo switchScreen(Long examRecordDataId);

}
