package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamSpecialSettingsType;
import cn.com.qmth.examcloud.api.commons.enums.ExamStageStartExamStatus;
import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetCoursesByIdListReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetCoursesByIdListResp;
import cn.com.qmth.examcloud.core.oe.student.api.request.*;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcExamScoreResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.CalcFaceBiopsyResultResp;
import cn.com.qmth.examcloud.core.oe.student.api.response.CheckPaperInExamResp;
import cn.com.qmth.examcloud.core.oe.student.base.utils.QuestionTypeUtil;
import cn.com.qmth.examcloud.core.oe.student.bean.client.CourseInfo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamFaceLiveVerifyRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamFaceLivenessVerifyRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.FaceBiopsyRepo;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamFaceLiveVerifyEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamFaceLivenessVerifyEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.entity.FaceBiopsyEntity;
import cn.com.qmth.examcloud.core.oe.student.dao.enums.FaceLiveVerifyStatus;
import cn.com.qmth.examcloud.core.oe.student.dao.enums.FaceVerifyResult;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordDataService;
import cn.com.qmth.examcloud.core.oe.student.service.ExamRecordQuestionsService;
import cn.com.qmth.examcloud.examwork.api.ExamStageCloudService;
import cn.com.qmth.examcloud.examwork.api.request.ModifyExamStageStartExamStatusReq;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.CourseCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.QuestionAnswerCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamRecordStatus;
import cn.com.qmth.examcloud.support.enums.FaceBiopsyScheme;
import cn.com.qmth.examcloud.support.enums.IsSuccess;
import cn.com.qmth.examcloud.support.enums.SyncStatus;
import cn.com.qmth.examcloud.support.examing.ExamQuestion;
import cn.com.qmth.examcloud.support.examing.ExamRecordData;
import cn.com.qmth.examcloud.support.examing.ExamRecordQuestions;
import cn.com.qmth.examcloud.support.examing.ExamingSession;
import cn.com.qmth.examcloud.support.helper.FaceBiopsyHelper;
import cn.com.qmth.examcloud.support.redis.RedisKeyHelper;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author chenken
 * @date 2018/8/15 11:16
 * @company QMTH
 * @description 考试记录数据服务实现
 */
@Service("examRecordDataService")
public class ExamRecordDataServiceImpl implements ExamRecordDataService {

    private static final Logger log = LoggerFactory.getLogger(ExamRecordDataServiceImpl.class);

    @Autowired
    private RedisClient redisClient;

    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @Autowired
    private FaceBiopsyRepo faceBiopsyRepo;

    @Autowired
    private ExamFaceLivenessVerifyRepo examFaceLivenessVerifyRepo;

    @Autowired
    private ExamFaceLiveVerifyRepo examFaceLiveVerifyRepo;

    @Autowired
    private ExamRecordQuestionsService examRecordQuestionsService;

    @Autowired
    private ExamStageCloudService examStageCloudService;

    @Autowired
    private CourseCloudService courseCloudService;

    @Transactional
    @Override
    public ExamRecordData createExamRecordData(ExamingSession examingSession, ExamSettingsCacheBean examBean,
                                               CourseCacheBean courseBean, String basePaperId, boolean isFullyObjective,
                                               Long examStageId, Integer examStageOrder) {
        ExamRecordDataEntity examRecordData = new ExamRecordDataEntity();
        examRecordData.setExamId(examBean.getId());
        examRecordData.setExamType(ExamType.valueOf(examBean.getExamType()));

        examRecordData.setExamStudentId(examingSession.getExamStudentId());
        examRecordData.setStudentId(examingSession.getStudentId());
        examRecordData.setOrgId(examingSession.getOrgId());
        examRecordData.setRootOrgId(examingSession.getRootOrgId());

        examRecordData.setCourseId(courseBean.getId());
        examRecordData.setBasePaperId(basePaperId);

        examRecordData.setPaperType(examingSession.getPaperType());

        examRecordData.setEnterExamTime(new Date());
        examRecordData.setIsContinued(false);
        examRecordData.setContinuedCount(0);
        examRecordData.setFaceSuccessCount(0);
        examRecordData.setFaceTotalCount(0);
        examRecordData.setFaceFailedCount(0);
        examRecordData.setFaceStrangerCount(0);
        examRecordData.setExamRecordStatus(ExamRecordStatus.EXAM_ING);
        examRecordData.setSyncStatus(SyncStatus.UNSYNC);
        examRecordData.setIsAllObjectivePaper(isFullyObjective);
        examRecordData.setIsIllegality(false);
        examRecordData.setIsWarn(false);
        examRecordData.setIsAudit(false);
        examRecordData.setExamStageId(examStageId);
        examRecordData.setExamStageOrder(examStageOrder);
        examRecordDataRepo.save(examRecordData);

        //更新场次的开考状态
        if (null != examStageId) {
            if (examBean.getSpecialSettingsEnabled()
                    && ExamSpecialSettingsType.STAGE_BASED == examBean.getSpecialSettingsType()) {
                //更改考务中的考试状态 TODO 高并发下此处可能会影响开考的性能
                ModifyExamStageStartExamStatusReq req = new ModifyExamStageStartExamStatusReq();
                req.setExamStageId(examStageId);
                req.setExamStageStartExamStatus(ExamStageStartExamStatus.STARTED.name());
                examStageCloudService.modifyExamStageStartExamStatus(req);
            }
        }

        ExamRecordData bean = ofExamRecordData(examRecordData);
        bean.setExceedMaxSwitchScreenCount(false);

        //存入redis
        saveExamRecordDataCache(examRecordData.getId(), bean);

        log.warn("createExamRecordData success! studentId:{}, examRecordDataId:{}", examRecordData.getStudentId(), examRecordData.getId());
        return bean;
    }

    @Override
    public void saveExamRecordDataCache(Long examRecordDataId, ExamRecordData data) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.set(key, data, 2592000);
    }

    @Override
    public ExamRecordData getExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        return redisClient.get(key, ExamRecordData.class);
    }

    @Override
    public void deleteExamRecordDataCache(Long examRecordDataId) {
        String key = RedisKeyHelper.getBuilder().examRecordDataKey(examRecordDataId);
        redisClient.delete(key);
    }

    private ExamRecordData ofExamRecordData(ExamRecordDataEntity entity) {
        ExamRecordData bean = new ExamRecordData();

        bean.setId(entity.getId());
        bean.setRootOrgId(entity.getRootOrgId());
        bean.setOrgId(entity.getOrgId());
        bean.setCourseId(entity.getCourseId());
        bean.setStudentId(entity.getStudentId());
        bean.setExamStudentId(entity.getExamStudentId());

        bean.setExamId(entity.getExamId());
        bean.setExamType(entity.getExamType());
        bean.setExamStageId(entity.getExamStageId());
        bean.setExamStageOrder(entity.getExamStageOrder());
        bean.setBasePaperId(entity.getBasePaperId());
        bean.setPaperType(entity.getPaperType());

        bean.setStartTime(entity.getStartTime());
        bean.setEndTime(entity.getEndTime());
        bean.setEnterExamTime(entity.getEnterExamTime());
        bean.setCleanTime(entity.getCleanTime());
        bean.setUsedExamTime(entity.getUsedExamTime());
        // bean.setLastActiveTime(entity.getLastActiveTime());

        bean.setIsContinued(entity.getIsContinued());
        bean.setIsExceed(entity.getIsExceed());
        bean.setContinuedCount(entity.getContinuedCount());
        // bean.setContinuedTime(entity.getContinuedTime());

        bean.setExamRecordStatus(entity.getExamRecordStatus());
        bean.setIsWarn(entity.getIsWarn());
        bean.setIsAudit(entity.getIsAudit());
        bean.setIsIllegality(entity.getIsIllegality());

        bean.setFaceVerifyResult(entity.getFaceVerifyResult());
        bean.setFaceTotalCount(entity.getFaceTotalCount());
        bean.setFaceSuccessCount(entity.getFaceSuccessCount());
        bean.setFaceFailedCount(entity.getFaceFailedCount());
        bean.setFaceStrangerCount(entity.getFaceStrangerCount());
        // bean.setFaceSuccessPercent(entity.getFaceSuccessPercent());
        // bean.setBaiduFaceLivenessSuccessPercent(entity.getBaiduFaceLivenessSuccessPercent());

        bean.setTotalScore(entity.getTotalScore());
        bean.setObjectiveScore(entity.getObjectiveScore());
        bean.setObjectiveAccuracy(entity.getObjectiveAccuracy());
        bean.setSubjectiveScore(entity.getSubjectiveScore());
        bean.setSuccPercent(entity.getSuccPercent());
        bean.setIsAllObjectivePaper(entity.getIsAllObjectivePaper());

        bean.setSyncStatus(entity.getSyncStatus());
        bean.setQuestionCount(entity.getQuestionCount());
        bean.setHandInExamType(entity.getHandInExamType());
        // bean.setSwitchScreenCount(0);
        // bean.setExceedMaxSwitchScreenCount(false);

        return bean;
    }

    @Override
    public List<Long> getExamRecordDataIds(GetExamRecordDataIdsReq req) {
        Long startId = req.getStartId();
        Long batchNum = req.getBatchNum();
        Integer size = req.getSize();
        if (startId == null) {
            throw new StatusException("1001", "startId 不能为空");
        }
        if (batchNum == null) {
            throw new StatusException("1002", "batchNum 不能为空");
        }
        if (size == null) {
            throw new StatusException("1003", "size 不能为空");
        }
        if (size > 200) {
            throw new StatusException("1004", "size 最大为200");
        }
        List<Long> ids = new ArrayList<Long>();
        List<ExamRecordDataEntity> list = examRecordDataRepo.getLimitExamRecordDataList(batchNum, startId, size);
        if (list != null && list.size() != 0) {
            for (ExamRecordDataEntity e : list) {
                ids.add(e.getId());
            }
        }
        return ids;
    }

    @Transactional
    @Override
    public void updateExamRecordDataBatchNum(UpdateExamRecordDataBatchNumReq req) {
        List<Long> ids = req.getIds();
        Long batchNum = req.getBatchNum();
        if (ids == null) {
            throw new StatusException("2001", "ids 不能为空");
        }
        if (batchNum == null) {
            throw new StatusException("2002", "batchNum 不能为空");
        }
        for (Long id : ids) {
            examRecordDataRepo.updateBatchNumById(batchNum, id);
        }
    }

    /**
     * 计算活体检测结果
     *
     * @param req
     * @return
     */
    @Override
    public CalcFaceBiopsyResultResp calcFaceBiopsyResult(CalcFaceBiopsyResultReq req) {
        ExamRecordData examRecordData = this.getExamRecordDataCache(req.getExamRecordDataId());

        //如果活检结果不为空，且活检失败，则判定为违纪有警告
        if (null != examRecordData.getFaceVerifyResult()) {
            if (IsSuccess.FAILED == examRecordData.getFaceVerifyResult()) {
                return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
            }
            return new CalcFaceBiopsyResultResp();
        }

        //如果考试记录中的活检记录为空，需要再次计算一次，并更新到考试记录中
        else {
            Long examId = examRecordData.getExamId();
            Long rootOrgId = examRecordData.getRootOrgId();
            Long studentId = examRecordData.getStudentId();

            //是否进行活体检测
            if (FaceBiopsyHelper.isFaceVerify(rootOrgId, examId, studentId)) {
                FaceBiopsyScheme faceBiopsyScheme = FaceBiopsyHelper.getFaceBiopsyScheme(rootOrgId);

                if (FaceBiopsyScheme.FACE_CLIENT == faceBiopsyScheme) {
                    // C端活体检测方案
                    List<ExamFaceLiveVerifyEntity> entities = examFaceLiveVerifyRepo.findByExamRecordDataId(examRecordData.getId());
                    if (CollectionUtils.isEmpty(entities)) {
                        return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
                    }

                    // 最后一次活检成功，则认为成功
                    ExamFaceLiveVerifyEntity entity = entities.stream().max((a, b) -> {
                        if (a.getId() > b.getId()) {
                            return 1;
                        }
                        return -1;
                    }).get();

                    if (FaceLiveVerifyStatus.SUCCESS == entity.getStatus()) {
                        return new CalcFaceBiopsyResultResp(IsSuccess.SUCCESS);
                    }

                    return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
                } else if (faceBiopsyScheme == FaceBiopsyScheme.FACE_MOTION) {
                    // Electron Client 自研活体检测方案
                    FaceBiopsyEntity faceBiopsy = faceBiopsyRepo.findByExamRecordDataId(examRecordData.getId());

                    //如果活检结果最终为true,只需要更新活检状态即可
                    if (faceBiopsy != null && faceBiopsy.getResult() != null && faceBiopsy.getResult()) {
                        return new CalcFaceBiopsyResultResp(IsSuccess.SUCCESS);
                    }

                    return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
                } else {
                    // FaceID活体检测方案
                    List<ExamFaceLivenessVerifyEntity> faceVerifies = examFaceLivenessVerifyRepo.findByExamRecordDataIdOrderById(examRecordData.getId());

                    if (null != faceVerifies && faceVerifies.size() > 0) {
                        ExamFaceLivenessVerifyEntity latestFaceVerify = faceVerifies.get(faceVerifies.size() - 1);
                        //最后一次活检成功，则认为成功，并更新考试记录相关属性
                        if (latestFaceVerify != null && latestFaceVerify.getVerifyResult() != null &&
                                latestFaceVerify.getVerifyResult() == FaceVerifyResult.VERIFY_SUCCESS) {

                            return new CalcFaceBiopsyResultResp(IsSuccess.SUCCESS);
                        }
                    }

                    return new CalcFaceBiopsyResultResp(true, true, IsSuccess.FAILED);
                }
            }

            return new CalcFaceBiopsyResultResp();
        }
    }

    /**
     * 计算客观题总得分和答题正确数
     */
    @Override
    public CalcExamScoreResp calcExamScore(CalcExamScoreReq req) {
        long startTime = System.currentTimeMillis();
        ExamRecordQuestions examRecordQuestions = examRecordQuestionsService.getExamRecordQuestions(req.getExamRecordDataId());

        // todo NPE

        // 考生作答记录明细
        List<ExamQuestion> examQuestions = examRecordQuestions.getExamQuestions();

        for (ExamQuestion examQuestion : examQuestions) {
            if (QuestionTypeUtil.isObjectiveQuestion(examQuestion.getQuestionType())) {
                // 如果客观题的标准答案为空，则更新标准答案
                if (StringUtils.isEmpty(examQuestion.getCorrectAnswer())) {
                    this.updateCorrectAnswer(req.getExamRecordDataId(), examQuestions, examQuestion);
                }
            }
        }

        double objectiveScore = 0d;//客观题总分
        int totalObjective = 0;//客观题总数
        int totalRightObjective = 0;//答题正确数

        for (int i = 0; i < examQuestions.size(); i++) {
            ExamQuestion examQuestion = examQuestions.get(i);
            if (!QuestionTypeUtil.isObjectiveQuestion(examQuestion.getQuestionType())) {
                // 跳过主观题
                continue;
            }

            totalObjective++;

            // 如果学生作答正确，则客观分累加，答题数累加
            if (examQuestion.getStudentAnswer() != null && examQuestion.getCorrectAnswer() != null
                    // && QuestionOptionHelper.isEqualAnswer(examQuestion.getCorrectAnswer(), examQuestion.getStudentAnswer())) {
                    && examQuestion.getStudentAnswer().equals(examQuestion.getCorrectAnswer())) {

                double questionScore = examQuestion.getQuestionScore();
                BigDecimal bigObjectiveScore = new BigDecimal(Double.toString(objectiveScore));
                BigDecimal bigQuestionScore = new BigDecimal(Double.toString(questionScore));

                objectiveScore = bigQuestionScore.add(bigObjectiveScore).doubleValue();
                totalRightObjective++;
            }
        }

        // 计算客观题答题正确率
        double objectiveAccuracy = 0;
        if (totalRightObjective > 0 && totalObjective > 0) {
            objectiveAccuracy = Double.parseDouble(new DecimalFormat("#.00").format(totalRightObjective * 100D / totalObjective));
        }

        CalcExamScoreResp resp = new CalcExamScoreResp();
        resp.setObjectiveScore(objectiveScore);
        resp.setObjectiveAccuracy(objectiveAccuracy);
        //交卷时,总分=客观分得分
        resp.setTotalScore(objectiveScore);

        long cost = System.currentTimeMillis() - startTime;
        log.warn("calcExamScore examRecordDataId:{}, totalQuestion:{}, totalObjective:{}, cost:{}ms",
                req.getExamRecordDataId(), examQuestions.size(), totalObjective, cost);
        return resp;
    }

    /**
     * 更新部分考试记录数据
     */
    @Transactional
    @Override
    public void updatePartialExamRecord(UpdatePartialExamRecordReq req) {
        if (null == req.getId()) {
            throw new StatusException("300001", "考试记录id不允许为空");
        }

        if (StringUtils.isNotEmpty(req.getSyncStatus())) {
            SyncStatus status;

            try {
                status = SyncStatus.valueOf(req.getSyncStatus());
            } catch (IllegalArgumentException e) {
                throw new StatusException("300002", "同步状态值不正确");
            }

            examRecordDataRepo.updateSyncStatusById(status, req.getId());
        }
    }

    @Override
    public CourseInfo getCourseInfo(Long examRecordDataId) {
        Long courseId = examRecordDataRepo.findCourseIdByExamRecordDataId(examRecordDataId);
        if (courseId == null) {
            throw new StatusException("考试记录不存在！");
        }

        GetCoursesByIdListReq req = new GetCoursesByIdListReq();
        req.setCourseIdList(Lists.newArrayList(courseId));
        GetCoursesByIdListResp resp = courseCloudService.getCoursesByIdList(req);
        if (CollectionUtils.isEmpty(resp.getCourseList())) {
            throw new StatusException("课程不存在！");
        }

        CourseBean bean = resp.getCourseList().get(0);

        CourseInfo info = new CourseInfo();
        info.setCourseId(bean.getId());
        info.setCourseCode(bean.getCode());
        info.setCourseName(bean.getName());
        return info;
    }

    /**
     * 更新客观题答案
     */
    private void updateCorrectAnswer(Long examRecordDataId, List<ExamQuestion> examQuestions, ExamQuestion examQuestion) {
        // 按题号由小到大提取相同题ID的小题列表（注：套题的子题ID相同）
        List<ExamQuestion> questionUnits = examQuestions.stream().
                filter(e -> e.getQuestionId().equals(examQuestion.getQuestionId())).
                sorted(Comparator.comparingInt(ExamQuestion::getOrder))
                .collect(Collectors.toList());

        // 获取当前题的正确答案
        QuestionAnswerCacheBean questionAnswerCache = CacheHelper.getQuestionAnswer(examQuestion.getQuestionId());
        List<String> rightAnswers = questionAnswerCache.getRightAnswers();

        if (CollectionUtils.isEmpty(rightAnswers)) {
            log.warn("updateCorrectAnswer rightAnswers is empty, questionId = {}", examQuestion.getQuestionId());
            return;
        }

        if (questionUnits.size() != rightAnswers.size()) {
            log.warn("updateCorrectAnswer rightAnswers incorrect, questionId = {}", examQuestion.getQuestionId());
            return;
        }

        // 循环保存所有小题单元
        for (int i = 0; i < questionUnits.size(); i++) {
            ExamQuestion curQuestion = questionUnits.get(i);
            if (!QuestionTypeUtil.isObjectiveQuestion(curQuestion.getQuestionType())) {
                // 跳过套题内主观题
                continue;
            }

            String rightAnswer = rightAnswers.get(i);

            // 更新当前小题的正确答案
            if (curQuestion.getOrder().equals(examQuestion.getOrder())) {
                examQuestion.setCorrectAnswer(rightAnswer);
            }

            curQuestion.setCorrectAnswer(rightAnswer);
            examRecordQuestionsService.saveExamQuestion(examRecordDataId, curQuestion.getOrder(), curQuestion);
        }
    }

    @Transactional
    @Override
    public void updateExamRecordStatus(UpdateExamRecordStatusReq req) {
        Long id = req.getId();
        String status = req.getExamRecordStatus();
        if (id == null) {
            throw new StatusException("3001", "id 不能为空");
        }
        ExamRecordStatus examRecordStatus = ExamRecordStatus.getByName(status);
        if (status == null || examRecordStatus == null) {
            throw new StatusException("3002", "status 不能为空");
        }
        examRecordDataRepo.updateExamRecordStatusById(examRecordStatus, new Date(), id);
    }

    @Override
    public CheckPaperInExamResp checkPaperInExam(CheckPaperInExamReq req) {
        Long id = examRecordDataRepo.getRecordIdByPaperId(req.getBasePaperId());
        CheckPaperInExamResp res = new CheckPaperInExamResp();
        if (id == null) {
            res.setInExam(false);
        } else {
            res.setInExam(true);
        }
        return res;
    }

}
