package cn.com.qmth.examcloud.core.oe.student.service.impl;

import cn.com.qmth.examcloud.core.oe.student.service.TempConfigService;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import com.google.common.collect.Lists;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class TempConfigServiceImpl implements TempConfigService {

    private static final Logger log = LoggerFactory.getLogger(TempConfigServiceImpl.class);

    private static final String DISABLED_ORG_ID_CACHE_KEY = "$DISABLED_ORG_ID";

    @Autowired
    private RedisClient redisClient;

    /**
     * 获取-被禁用旧版本API的学校列表
     */
    @Override
    public List<Long> getDisabledRootOrgIdsForOldApi() {
        List<Long> rootOrgIds = redisClient.get(DISABLED_ORG_ID_CACHE_KEY, List.class);
        if (rootOrgIds == null) {
            return Lists.newArrayList();
        }

        return rootOrgIds;
    }

    /**
     * 修改-被禁用旧版本API的学校列表
     */
    @Override
    public void updateDisabledRootOrgIdsForOldApi(List<Long> rootOrgIds) {
        if (CollectionUtils.isEmpty(rootOrgIds)) {
            redisClient.delete(DISABLED_ORG_ID_CACHE_KEY);
            log.warn("updateDisabledRootOrgIdsForOldApi ids is empty");
        } else {
            redisClient.set(DISABLED_ORG_ID_CACHE_KEY, rootOrgIds);
            log.warn("updateDisabledRootOrgIdsForOldApi ids={}", rootOrgIds);
        }
    }

    /**
     * 判断学校是否被禁用旧版本API
     */
    @Override
    public boolean checkDisabledForOldApi(Long rootOrgId) {
        List<Long> rootOrgIds = this.getDisabledRootOrgIdsForOldApi();
        return rootOrgIds.contains(rootOrgId);
    }

}
